@@ -18,6 +18,8 @@
 
 """Converters, etc for going between Bazaar and Git ids."""
 
+from __future__ import absolute_import
+
 import base64
 import stat
 
@@ -38,7 +40,10 @@ from bzrlib.foreign import (
 from bzrlib.revision import (
     NULL_REVISION,
     )
-from bzrlib.plugins.git.errors import NoPushSupport
+from bzrlib.plugins.git.errors import (
+    NoPushSupport,
+    UnknownCommitExtra,
+    )
 from bzrlib.plugins.git.hg import (
     format_hg_metadata,
     extract_hg_metadata,
@@ -325,6 +330,8 @@ class BzrGitMapping(foreign.VcsMapping):
         """
         if commit is None:
             raise AssertionError("Commit object can't be None")
+        if commit.extra:
+            raise UnknownCommitExtra(commit, [item[0] for item in commit.extra])
         rev = ForeignRevision(commit.id, self,
                 self.revision_id_foreign_to_bzr(commit.id))
         rev.git_metadata = None
@@ -483,10 +490,9 @@ foreign_vcs_git = ForeignGit()
 default_mapping = mapping_registry.get_default()()
 
 
-def symlink_to_blob(entry):
+def symlink_to_blob(symlink_target):
     from dulwich.objects import Blob
     blob = Blob()
-    symlink_target = entry.symlink_target
     if type(symlink_target) == unicode:
         symlink_target = symlink_target.encode('utf-8')
     blob.data = symlink_target
@@ -546,10 +552,11 @@ def entry_mode(entry):
     return object_mode(entry.kind, entry.executable)
 
 
-def directory_to_tree(entry, lookup_ie_sha1, unusual_modes, empty_file_name):
+def directory_to_tree(children, lookup_ie_sha1, unusual_modes, empty_file_name,
+                      allow_empty=False):
     """Create a Git Tree object from a Bazaar directory.
 
-    :param entry: Inventory entry
+    :param children: Children inventory entries
     :param lookup_ie_sha1: Lookup the Git SHA1 for a inventory entry
     :param unusual_modes: Dictionary with unusual file modes by file ids
     :param empty_file_name: Name to use for dummy files in empty directories,
@@ -557,8 +564,8 @@ def directory_to_tree(entry, lookup_ie_sha1, unusual_modes, empty_file_name):
     """
     from dulwich.objects import Blob, Tree
     tree = Tree()
-    for name, value in entry.children.iteritems():
-        ie = entry.children[name]
+    for name, value in children.iteritems():
+        ie = children[name]
         try:
             mode = unusual_modes[ie.file_id]
         except KeyError:
@@ -566,7 +573,7 @@ def directory_to_tree(entry, lookup_ie_sha1, unusual_modes, empty_file_name):
         hexsha = lookup_ie_sha1(ie)
         if hexsha is not None:
             tree.add(name.encode("utf-8"), mode, hexsha)
-    if entry.parent_id is not None and len(tree) == 0:
+    if not allow_empty and len(tree) == 0:
         # Only the root can be an empty tree
         if empty_file_name is not None:
             tree.add(empty_file_name, stat.S_IFREG | 0644, Blob().id)
@@ -598,6 +605,9 @@ class GitFileIdMap(object):
         self.paths = None
         self.mapping = mapping
 
+    def all_file_ids(self):
+        return self.file_ids.values()
+
     def set_file_id(self, path, file_id):
         assert type(path) is str
         assert type(file_id) is str
