@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     config,
     debug,
@@ -78,6 +80,9 @@ import os
 import tempfile
 import urllib
 import urlparse
+
+# urlparse only supports a limited number of schemes by default
+
 urlparse.uses_netloc.extend(['git', 'git+ssh'])
 
 from dulwich.pack import load_pack_index
@@ -104,6 +109,11 @@ def split_git_url(url):
     return (host, port, username, path)
 
 
+class RemoteGitError(BzrError):
+
+    _fmt = "Remote server error: %(message)s"
+
+
 def parse_git_error(url, message):
     """Parse a remote git server error and return a bzr exception.
 
@@ -113,8 +123,13 @@ def parse_git_error(url, message):
     message = str(message).strip()
     if message.startswith("Could not find Repository "):
         return NotBranchError(url, message)
+    if message == "HEAD failed to update":
+        base_url, _ = urlutils.split_segment_parameters(url)
+        raise BzrError(
+            ("Unable to update remote HEAD branch. To update the master "
+             "branch, specify the URL %s,branch=master.") % base_url)
     # Don't know, just return it to the user as-is
-    return BzrError(message)
+    return RemoteGitError(message)
 
 
 class GitSmartTransport(Transport):
@@ -127,10 +142,6 @@ class GitSmartTransport(Transport):
             trace.mutter('host: %r, user: %r, port: %r, path: %r',
                          self._host, self._username, self._port, self._path)
         self._client = _client
-        if "," in self._path and bzrlib_version < (2, 5, 0):
-            trace.warning(
-                "ignoring parameters %r, not supported in bzr < 2.5.",
-                self._path.rsplit(",", 1)[1])
         self._stripped_path = self._path.rsplit(",", 1)[0]
 
     def external_url(self):
@@ -237,10 +248,11 @@ class RemoteGitDir(GitDir):
         except GitProtocolError, e:
             raise parse_git_error(self.transport.external_url(), e)
 
+    def _get_default_ref(self):
+        return "refs/heads/master"
+
     def destroy_branch(self, name=None):
         refname = self._get_selected_ref(name)
-        if refname is None:
-            refname = "HEAD"
         def get_changed_refs(old_refs):
             ret = dict(old_refs)
             if not refname in ret:
@@ -375,7 +387,7 @@ class RemoteGitControlDirFormat(GitControlDirFormat):
         elif urlparse.urlsplit(transport.external_url())[0] in ("http", "https"):
             def get_client(thin_packs=False):
                 return BzrGitHttpClient(transport, thin_packs=thin_packs)
-            client_path = transport._path
+            client_path, _ = urlutils.split_segment_parameters(transport._path)
         else:
             raise NotBranchError(transport.base)
         return RemoteGitDir(transport, self, get_client, client_path)
@@ -448,6 +460,9 @@ class RemoteGitRepository(GitRepository):
     def get_revisions(self, revids):
         raise GitSmartRemoteNotSupported(self.get_revisions, self)
 
+    def has_revisions(self, revids):
+        raise GitSmartRemoteNotSupported(self.get_revisions, self)
+
 
 class RemoteGitTagDict(GitTags):
 
@@ -476,9 +491,6 @@ class RemoteGitBranch(GitBranch):
     def control_url(self):
         return self.base
 
-    def revision_history(self):
-        raise GitSmartRemoteNotSupported(self.revision_history, self)
-
     def revision_id_to_revno(self, revision_id):
         raise GitSmartRemoteNotSupported(self.revision_id_to_revno, self)
 
@@ -490,7 +502,7 @@ class RemoteGitBranch(GitBranch):
         if self._sha is not None:
             return self._sha
         refs = self.bzrdir.get_refs_container()
-        name = branch_name_to_ref(self.name, "HEAD")
+        name = branch_name_to_ref(self.name)
         try:
             self._sha = refs[name]
         except KeyError:
