@@ -45,6 +45,8 @@ This extra metadata is stored in so-called "supplements":
   * TreeSupplement
 """
 
+from __future__ import absolute_import
+
 from bzrlib import osutils
 
 from cStringIO import StringIO
@@ -146,7 +148,11 @@ def inject_bzr_metadata(message, commit_supplement, encoding):
 
 
 def serialize_fileid_map(file_ids):
-    """Serialize a file id map."""
+    """Serialize a fileid map.
+
+    :param file_ids: Path -> fileid map
+    :return: Serialized fileid map, as sequence of chunks
+    """
     lines = []
     for path in sorted(file_ids.keys()):
         lines.append("%s\0%s\n" % (path, file_ids[path]))
@@ -154,7 +160,11 @@ def serialize_fileid_map(file_ids):
 
 
 def deserialize_fileid_map(filetext):
-    """Deserialize a file id map."""
+    """Deserialize a file id map.
+
+    :param file: File
+    :return: Fileid map (path -> fileid)
+    """
     ret = {}
     f = StringIO(filetext)
     lines = f.readlines()
