@@ -17,19 +17,25 @@
 
 from dulwich.objects import (
     Blob,
+    Tag,
     Tree,
+    S_IFGITLINK,
     )
 from dulwich.repo import (
     Repo as GitRepo,
     )
 import os
 import stat
+import time
 
 from bzrlib import (
     knit,
     osutils,
     versionedfile,
     )
+from bzrlib.branch import (
+    Branch,
+    )
 from bzrlib.bzrdir import (
     BzrDir,
     )
@@ -46,6 +52,7 @@ from bzrlib.tests import (
 from bzrlib.plugins.git.fetch import (
     import_git_blob,
     import_git_tree,
+    import_git_submodule,
     )
 from bzrlib.plugins.git.mapping import (
     BzrGitMappingv1,
@@ -56,11 +63,11 @@ from bzrlib.plugins.git.tests import (
     )
 
 
-class RepositoryFetchTests:
+class RepositoryFetchTests(object):
 
     def make_git_repo(self, path):
         os.mkdir(path)
-        GitRepo.init(path)
+        return GitRepo.init(os.path.abspath(path))
 
     def clone_git_repo(self, from_url, to_url, revision_id=None):
         oldrepos = self.open_git_repo(from_url)
@@ -88,7 +95,8 @@ class RepositoryFetchTests:
         path, gitsha = self.make_onerev_branch()
         oldrepo = self.open_git_repo(path)
         newrepo = self.clone_git_repo(path, "f")
-        self.assertEquals([oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha)], newrepo.all_revision_ids())
+        revid = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha)
+        self.assertEquals([revid], newrepo.all_revision_ids())
 
     def test_single_rev_specific(self):
         path, gitsha = self.make_onerev_branch()
@@ -136,14 +144,12 @@ class RepositoryFetchTests:
         tree1 = newrepo.revision_tree(revid1)
         tree2 = newrepo.revision_tree(revid2)
         fileid = tree1.path2id("mylink")
-        ie1 = tree1.inventory[fileid]
-        ie2 = tree2.inventory[fileid]
-        self.assertEquals(revid1, ie1.revision)
-        self.assertEquals("directory", ie1.kind)
-        self.assertEquals(None, ie1.symlink_target)
-        self.assertEquals(revid2, ie2.revision)
-        self.assertEquals("symlink", ie2.kind)
-        self.assertEquals("target/", ie2.symlink_target)
+        self.assertEquals(revid1, tree1.get_file_revision(fileid))
+        self.assertEquals("directory", tree1.kind(fileid))
+        self.assertEquals(None, tree1.get_symlink_target(fileid))
+        self.assertEquals(revid2, tree2.get_file_revision(fileid))
+        self.assertEquals("symlink", tree2.kind(fileid))
+        self.assertEquals("target/", tree2.get_symlink_target(fileid))
 
     def test_symlink_becomes_dir(self):
         self.make_git_repo("d")
@@ -164,14 +170,12 @@ class RepositoryFetchTests:
         tree1 = newrepo.revision_tree(revid1)
         tree2 = newrepo.revision_tree(revid2)
         fileid = tree1.path2id("mylink")
-        ie1 = tree1.inventory[fileid]
-        ie2 = tree2.inventory[fileid]
-        self.assertEquals(revid1, ie1.revision)
-        self.assertEquals("symlink", ie1.kind)
-        self.assertEquals("target/", ie1.symlink_target)
-        self.assertEquals(revid2, ie2.revision)
-        self.assertEquals("directory", ie2.kind)
-        self.assertEquals(None, ie2.symlink_target)
+        self.assertEquals(revid1, tree1.get_file_revision(fileid))
+        self.assertEquals("symlink", tree1.kind(fileid))
+        self.assertEquals("target/", tree1.get_symlink_target(fileid))
+        self.assertEquals(revid2, tree2.get_file_revision(fileid))
+        self.assertEquals("directory", tree2.kind(fileid))
+        self.assertEquals(None, tree2.get_symlink_target(fileid))
 
     def test_changing_symlink(self):
         self.make_git_repo("d")
@@ -192,12 +196,10 @@ class RepositoryFetchTests:
         tree1 = newrepo.revision_tree(revid1)
         tree2 = newrepo.revision_tree(revid2)
         fileid = tree1.path2id("mylink")
-        ie1 = tree1.inventory[fileid]
-        ie2 = tree2.inventory[fileid]
-        self.assertEquals(revid1, ie1.revision)
-        self.assertEquals("target", ie1.symlink_target)
-        self.assertEquals(revid2, ie2.revision)
-        self.assertEquals("target/", ie2.symlink_target)
+        self.assertEquals(revid1, tree1.get_file_revision(fileid))
+        self.assertEquals("target", tree1.get_symlink_target(fileid))
+        self.assertEquals(revid2, tree2.get_file_revision(fileid))
+        self.assertEquals("target/", tree2.get_symlink_target(fileid))
 
     def test_executable(self):
         self.make_git_repo("d")
@@ -213,9 +215,9 @@ class RepositoryFetchTests:
         revid = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha)
         tree = newrepo.revision_tree(revid)
         self.assertTrue(tree.has_filename("foobar"))
-        self.assertEquals(True, tree.inventory[tree.path2id("foobar")].executable)
+        self.assertEquals(True, tree.is_executable(tree.path2id("foobar")))
         self.assertTrue(tree.has_filename("notexec"))
-        self.assertEquals(False, tree.inventory[tree.path2id("notexec")].executable)
+        self.assertEquals(False, tree.is_executable(tree.path2id("notexec")))
 
     def test_becomes_executable(self):
         self.make_git_repo("d")
@@ -232,9 +234,44 @@ class RepositoryFetchTests:
         revid = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha2)
         tree = newrepo.revision_tree(revid)
         self.assertTrue(tree.has_filename("foobar"))
-        ie = tree.inventory[tree.path2id("foobar")]
-        self.assertEquals(True, ie.executable)
-        self.assertEquals(revid, ie.revision)
+        fileid = tree.path2id("foobar")
+        self.assertEquals(True, tree.is_executable(fileid))
+        self.assertEquals(revid, tree.get_file_revision(fileid))
+
+    def test_into_stacked_on(self):
+        r = self.make_git_repo("d")
+        os.chdir("d")
+        bb = GitBranchBuilder()
+        bb.set_file(u"foobar", "foo\n", False)
+        mark1 = bb.commit("Somebody <somebody@someorg.org>", "mymsg1")
+        gitsha1 = bb.finish()[mark1]
+        os.chdir("..")
+        stacked_on = self.clone_git_repo("d", "stacked-on")
+        oldrepo = Repository.open("d")
+        revid1 = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha1)
+        self.assertEquals([revid1], stacked_on.all_revision_ids())
+        b = stacked_on.bzrdir.create_branch()
+        b.generate_revision_history(revid1)
+        self.assertEquals(b.last_revision(), revid1)
+        tree = self.make_branch_and_tree("stacked")
+        tree.branch.set_stacked_on_url(b.user_url)
+        os.chdir("d")
+        bb = GitBranchBuilder()
+        bb.set_file(u"barbar", "bar\n", False)
+        bb.set_file(u"foo/blie/bla", "bla\n", False)
+        mark2 = bb.commit("Somebody <somebody@someorg.org>", "mymsg2")
+        gitsha2 = bb.finish()[mark2]
+        revid2 = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha2)
+        os.chdir("..")
+        tree.branch.fetch(Branch.open("d"))
+        tree.branch.repository.check()
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEquals(
+            set([(revid2,)]),
+            tree.branch.repository.revisions.without_fallbacks().keys())
+        self.assertEquals(
+            set([revid1, revid2]),
+            set(tree.branch.repository.all_revision_ids()))
 
     def test_non_ascii_characters(self):
         self.make_git_repo("d")
@@ -250,6 +287,29 @@ class RepositoryFetchTests:
         tree = newrepo.revision_tree(revid)
         self.assertTrue(tree.has_filename(u"foőbar"))
 
+    def test_tagged_tree(self):
+        r = self.make_git_repo("d")
+        os.chdir("d")
+        bb = GitBranchBuilder()
+        bb.set_file("foobar", "fooll\nbar\n", False)
+        mark = bb.commit("Somebody <somebody@someorg.org>", "nextmsg")
+        marks = bb.finish()
+        gitsha = marks[mark]
+        tag = Tag()
+        tag.name = "sometag"
+        tag.tag_time = time.time()
+        tag.tag_timezone = 0
+        tag.tagger = "Somebody <somebody@example.com>"
+        tag.message = "Created tag pointed at tree"
+        tag.object = (Tree, r[gitsha].tree)
+        r.object_store.add_object(tag)
+        r["refs/tags/sometag"] = tag
+        os.chdir("..")
+        oldrepo = self.open_git_repo("d")
+        revid = oldrepo.get_mapping().revision_id_foreign_to_bzr(gitsha)
+        newrepo = self.clone_git_repo("d", "f")
+        self.assertEquals(set([revid]), set(newrepo.all_revision_ids()))
+
 
 class LocalRepositoryFetchTests(RepositoryFetchTests, TestCaseWithTransport):
 
@@ -339,7 +399,7 @@ class ImportObjects(TestCaseWithTransport):
         base_inv = Inventory()
         blob = Blob.from_string("bar1")
         tree = Tree()
-        tree.add(stat.S_IFREG | 0644, "foo", blob.id)
+        tree.add("foo", stat.S_IFREG | 0644, blob.id)
         objects = { blob.id: blob, tree.id: tree }
         ret, child_modes = import_git_tree(self._texts, self._mapping, "bla", "bla",
                 (None, tree.id), base_inv, None, "somerevid", [],
@@ -364,10 +424,10 @@ class ImportObjects(TestCaseWithTransport):
         base_inv = Inventory()
         blob = Blob.from_string("bar1")
         tree = Tree()
-        tree.add(stat.S_IFREG | 0664, "foo", blob.id)
+        tree.add("foo", stat.S_IFREG | 0664, blob.id)
         objects = { blob.id: blob, tree.id: tree }
-        ret, child_modes = import_git_tree(self._texts, self._mapping, "bla", "bla",
-                (None, tree.id), base_inv, None, "somerevid", [],
+        ret, child_modes = import_git_tree(self._texts, self._mapping,
+            "bla", "bla", (None, tree.id), base_inv, None, "somerevid", [],
             objects.__getitem__, (None, stat.S_IFDIR), DummyStoreUpdater(),
             self._mapping.generate_file_id)
         self.assertEquals(child_modes, { "bla/foo": stat.S_IFREG | 0664 })
@@ -376,7 +436,7 @@ class ImportObjects(TestCaseWithTransport):
         base_inv = Inventory(root_id=None)
         blob = Blob.from_string("bar")
         tree = Tree()
-        tree.add(0100755, "foo", blob.id)
+        tree.add("foo", 0100755, blob.id)
         objects = { blob.id: blob, tree.id: tree }
         ret, child_modes = import_git_tree(self._texts, self._mapping, "", "",
                 (None, tree.id), base_inv, None, "somerevid", [],
@@ -393,3 +453,23 @@ class ImportObjects(TestCaseWithTransport):
         ie = ret[1][3]
         self.assertEquals("file", ie.kind)
         self.assertEquals(True, ie.executable)
+
+    def test_directory_converted_to_submodule(self):
+        base_inv = Inventory()
+        base_inv.add_path("foo", "directory")
+        base_inv.add_path("foo/bar", "file")
+        othertree = Blob.from_string("someotherthing")
+        blob = Blob.from_string("bar")
+        tree = Tree()
+        tree.add("bar", 0160000, blob.id)
+        objects = { tree.id: tree }
+        ret, child_modes = import_git_submodule(self._texts, self._mapping, "foo", "foo",
+                (tree.id, othertree.id), base_inv, base_inv.root.file_id, "somerevid", [],
+                objects.__getitem__, (stat.S_IFDIR | 0755, S_IFGITLINK), DummyStoreUpdater(),
+                self._mapping.generate_file_id)
+        self.assertEquals(child_modes, {})
+        self.assertEquals(2, len(ret))
+        self.assertEquals(ret[0], ("foo/bar", None, base_inv.path2id("foo/bar"), None))
+        self.assertEquals(ret[1][:3], ("foo", "foo", self._mapping.generate_file_id("foo")))
+        ie = ret[1][3]
+        self.assertEquals(ie.kind, "tree-reference")
