@@ -16,9 +16,12 @@
 
 """A Git repository implementation that uses a Bazaar transport."""
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 import os
+import urllib
 
 from dulwich.errors import (
     NotGitRepository,
@@ -87,7 +90,7 @@ class TransportRefsContainer(RefsContainer):
         keys = set()
         try:
             iter_files = self.transport.clone(base).iter_files_recursive()
-            keys.update(("%s/%s" % (base, refname)).strip("/") for 
+            keys.update(("%s/%s" % (base, urllib.unquote(refname))).strip("/") for 
                     refname in iter_files if check_ref_format("%s/%s" % (base, refname)))
         except (TransportNotPossible, NoSuchFile):
             pass
@@ -107,7 +110,7 @@ class TransportRefsContainer(RefsContainer):
         try:
             iter_files = list(self.transport.clone("refs").iter_files_recursive())
             for filename in iter_files:
-                refname = "refs/%s" % filename
+                refname = "refs/%s" % urllib.unquote(filename)
                 if check_ref_format(refname):
                     keys.add(refname)
         except (TransportNotPossible, NoSuchFile):
@@ -182,6 +185,7 @@ class TransportRefsContainer(RefsContainer):
             f = self.transport.get(name)
         except NoSuchFile:
             return None
+        f = StringIO(f.read())
         try:
             header = f.read(len(SYMREF))
             if header == SYMREF:
@@ -306,6 +310,12 @@ class TransportRepo(BaseRepo):
         if refs_text is not None:
             from dulwich.repo import InfoRefsContainer # dulwich >= 0.8.2
             refs_container = InfoRefsContainer(StringIO(refs_text))
+            try:
+                head = TransportRefsContainer(self._controltransport).read_loose_ref("HEAD")
+            except KeyError:
+                pass
+            else:
+                refs_container._refs["HEAD"] = head
         else:
             refs_container = TransportRefsContainer(self._controltransport)
         super(TransportRepo, self).__init__(object_store, 
@@ -422,8 +432,7 @@ class TransportObjectStore(PackBasedObjectStore):
             return []
         ret = []
         try:
-            for l in f.readlines():
-                l = l.rstrip("\n")
+            for l in f.read().splitlines():
                 if l[0] == "#":
                     continue
                 if os.path.isabs(l):
@@ -440,8 +449,7 @@ class TransportObjectStore(PackBasedObjectStore):
             return self.pack_transport.list_dir(".")
         else:
             ret = []
-            for line in f.readlines():
-                line = line.rstrip("\n")
+            for line in f.read().splitlines():
                 if not line:
                     continue
                 (kind, name) = line.split(" ", 1)
