@@ -17,6 +17,7 @@
 
 """An adapter between a Git index and a Bazaar Working Tree"""
 
+from __future__ import absolute_import
 
 from cStringIO import (
     StringIO,
@@ -166,6 +167,18 @@ class GitWorkingTree(workingtree.WorkingTree):
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         self.set_parent_ids([p for p, t in parents_list])
 
+    def iter_children(self, file_id):
+        dpath = self.id2path(file_id) + "/"
+        if dpath in self.index:
+            return
+        for path in self.index:
+            if not path.startswith(dpath):
+                continue
+            if "/" in path[len(dpath):]:
+                # Not a direct child but something further down
+                continue
+            yield self.path2id(path)
+
     def _index_add_entry(self, path, file_id, kind):
         assert self._lock_mode is not None
         assert isinstance(path, basestring)
@@ -414,6 +427,8 @@ class GitWorkingTree(workingtree.WorkingTree):
 
     @needs_read_lock
     def path2id(self, path):
+        if type(path) is list:
+            path = u"/".join(path)
         encoded_path = path.encode("utf-8")
         if self._is_versioned(encoded_path):
             return self._fileid_map.lookup_file_id(encoded_path)
@@ -537,7 +552,12 @@ class GitWorkingTree(workingtree.WorkingTree):
     def get_file_verifier(self, file_id, path=None, stat_value=None):
         if path is None:
             path = self.id2path(file_id)
-        return ("GIT", self.index[path][-2])
+        try:
+            return ("GIT", self.index[path][-2])
+        except KeyError:
+            if self._has_dir(path):
+                return ("GIT", None)
+            raise errors.NoSuchId(self, file_id)
 
     @needs_read_lock
     def get_file_sha1(self, file_id, path=None, stat_value=None):
